# mypy: ignore-errors
# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
# isort: skip_file
from .....schema_classes import DataHubAiConversationActorClass
from .....schema_classes import DataHubAiConversationActorTypeClass
from .....schema_classes import DataHubAiConversationContextClass
from .....schema_classes import DataHubAiConversationInfoClass
from .....schema_classes import DataHubAiConversationMessageClass
from .....schema_classes import DataHubAiConversationMessageContentClass
from .....schema_classes import DataHubAiConversationMessageTypeClass
from .....schema_classes import DataHubAiConversationOriginTypeClass


DataHubAiConversationActor = DataHubAiConversationActorClass
DataHubAiConversationActorType = DataHubAiConversationActorTypeClass
DataHubAiConversationContext = DataHubAiConversationContextClass
DataHubAiConversationInfo = DataHubAiConversationInfoClass
DataHubAiConversationMessage = DataHubAiConversationMessageClass
DataHubAiConversationMessageContent = DataHubAiConversationMessageContentClass
DataHubAiConversationMessageType = DataHubAiConversationMessageTypeClass
DataHubAiConversationOriginType = DataHubAiConversationOriginTypeClass

# fmt: on
