{
  "type": "record",
  "Aspect": {
    "name": "corpUserUsageFeatures"
  },
  "name": "CorpUserUsageFeatures",
  "namespace": "com.linkedin.pegasus2avro.metadata.search.features",
  "fields": [
    {
      "Searchable": {
        "fieldName": "userUsageTotalPast30DaysFeature",
        "fieldType": "COUNT"
      },
      "type": [
        "null",
        "long"
      ],
      "name": "userUsageTotalPast30Days",
      "default": null,
      "doc": "Total number of usage in the last 30 days"
    },
    {
      "Searchable": {
        "/*": {
          "fieldName": "platformUsageTotal",
          "fieldType": "OBJECT",
          "queryByDefault": false
        }
      },
      "type": [
        "null",
        {
          "type": "map",
          "values": "double"
        }
      ],
      "name": "userPlatformUsageTotalsPast30Days",
      "default": null,
      "doc": "usage broken down by platform so we can differentiate between top users of snowflake vs redshift\nkey is supposed to be platform urn"
    },
    {
      "Searchable": {
        "/*": {
          "fieldName": "platformUsagePercentage",
          "fieldType": "OBJECT",
          "queryByDefault": false
        }
      },
      "type": [
        "null",
        {
          "type": "map",
          "values": "double"
        }
      ],
      "name": "userPlatformUsagePercentilePast30Days",
      "default": null,
      "doc": "usage broken down by platform so we can differentiate between top users of snowflake vs redshift"
    },
    {
      "Searchable": {
        "fieldName": "userUsagePercentilePast30DaysFeature",
        "fieldType": "DOUBLE"
      },
      "type": [
        "null",
        "double"
      ],
      "name": "userUsagePercentilePast30Days",
      "default": null,
      "doc": "Platform usage total percentile for ranking users across all platforms"
    },
    {
      "type": [
        "null",
        {
          "type": "map",
          "values": "double"
        }
      ],
      "name": "userTopDatasetsByUsage",
      "default": null,
      "doc": "Gather what are the top datasets per user so we can create a welcome module\nWith links to the users top datasets with lineage (top 25)\nMap keys are dataset URN strings"
    }
  ]
}