{
  "type": "record",
  "Aspect": {
    "name": "formAssignmentStatus"
  },
  "name": "FormAssignmentStatus",
  "namespace": "com.linkedin.pegasus2avro.form",
  "fields": [
    {
      "type": {
        "type": "enum",
        "symbolDocs": {
          "COMPLETE": "Assets assignment is complete",
          "IN_PROGRESS": "Assets are currently being assigned"
        },
        "name": "AssignmentStatus",
        "namespace": "com.linkedin.pegasus2avro.form",
        "symbols": [
          "IN_PROGRESS",
          "COMPLETE"
        ]
      },
      "name": "status",
      "doc": "Notification settings for a form"
    },
    {
      "type": [
        "null",
        "long"
      ],
      "name": "timestamp",
      "default": null,
      "doc": "Timestamp this aspect was last updated at"
    }
  ]
}